/*
 * CoverFinder - Find and download cover images from Amazon
 * Copyright (C) 2007 - 2008 - Sven Salzwedel
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
#include <gtk/gtk.h>
#include <gio/gio.h>
#include <string.h>

#include "cf-app.h"
#include "cf-search.h"
#include "cf-provider.h"
#include "cf-provider-amazon.h"
#include "cf-search-result.h"
#include "cf-util.h"
#include "config.h"

G_DEFINE_TYPE (CfApp, cf_app, G_TYPE_OBJECT)

typedef struct
{
	CfApp *app;
	CfProvider *provider;
	CfSearch *search;
	guint flavor_id;
	GdkCursor *cursor;
} SearchThreadData;

static GtkTargetEntry targets[] = { { "text/uri-list", 0, 2 } };

static void cf_app_cb_notify_result (CfProvider *provider,
                                     CfSearchResult *result,
                                     gpointer user_data)
{
	GtkTreeIter iter;
	GtkListStore *model;
	gchar *desc, *pixbuf_path;
	CfApp *self = CF_APP (user_data);
	GdkPixbuf *unscaled_pixbuf, *pixbuf;
	GFile *image;
	GError *error = NULL;

	if (result == NULL) {
		return;
	}

	gdk_threads_enter ();
	model = GTK_LIST_STORE (gtk_tree_view_get_model (self->results_view));
	gtk_list_store_append (model, &iter);
	desc = cf_search_result_get_description (result);

	image = cf_search_result_get_image (result);
	if (image != NULL) {
		pixbuf_path = g_file_get_path (image);
		unscaled_pixbuf = gdk_pixbuf_new_from_file (pixbuf_path, &error);
		if (error == NULL) {
			gint width, height;
			gchar *tmp;

			width = gdk_pixbuf_get_width (unscaled_pixbuf);
			height = gdk_pixbuf_get_height (unscaled_pixbuf);

			tmp = desc;
			desc = g_strdup_printf ("%s\n\nSize: %i x %i", tmp, width, height);
			g_free (tmp);

			pixbuf = gdk_pixbuf_scale_simple (unscaled_pixbuf, 100, 100,
			                                  GDK_INTERP_HYPER);
			gtk_list_store_set (model, &iter, CF_RESULT_COLUMN_IMG, pixbuf, -1);
			g_object_unref (unscaled_pixbuf);
		} else {
			g_printerr ("%s", error->message);
			g_error_free (error);
		}
		g_free (pixbuf_path);
	}

	gtk_list_store_set (model, &iter, CF_RESULT_COLUMN_DESCRIPTION, desc,
	                    CF_RESULT_COLUMN_RESULT, result, -1);
	g_free (desc);
	gdk_threads_leave ();
}

static void cf_app_cb_notify_progress (CfProvider *provider,
                                       gdouble fraction,
                                       gchar *message,
                                       gpointer user_data)
{
	CfApp *self;

	gdk_threads_enter ();

	self = CF_APP (user_data);
	if (self->statusbar_message == NULL) {
		self->statusbar_message = g_strdup (message);
		gtk_statusbar_push (self->statusbar, self->statusbar_default_ctx,
		                    message);
	} else if (message == NULL) {
		g_free (self->statusbar_message);
		self->statusbar_message = NULL;
		gtk_statusbar_pop (self->statusbar, self->statusbar_default_ctx);
	} else if (!g_str_equal (self->statusbar_message, message)) {
		g_free (self->statusbar_message);
		gtk_statusbar_pop (self->statusbar, self->statusbar_default_ctx);
		self->statusbar_message = g_strdup (message);
		gtk_statusbar_push (self->statusbar, self->statusbar_default_ctx,
		                    message);
	}

	gtk_progress_bar_set_fraction (self->progress_bar, fraction);
	gdk_threads_leave ();
}

static void cf_app_finalize (GObject *object)
{
	if (G_OBJECT_CLASS (cf_app_parent_class)->finalize)
		G_OBJECT_CLASS (cf_app_parent_class)->finalize (object);
}

static void cf_app_class_init (CfAppClass *klass)
{
	GObjectClass *object_class = G_OBJECT_CLASS (klass);

	object_class->finalize = cf_app_finalize;
}

static void cf_app_save_selection (CfApp *self)
{
	GtkWidget *chooser;
	GtkWindow *parent;
	CfSearchResult *result;
	GtkTreeSelection *selection;
	GtkTreeIter iter;
	GtkTreeModel *model;
	GFile *source, *dest;
	gchar *dest_path;
	GError *error = NULL;

	parent = GTK_WINDOW (gtk_builder_get_object(self->builder, "CoverfinderWindow"));

	chooser = gtk_file_chooser_dialog_new ("Save Cover Image", parent,
	                                       GTK_FILE_CHOOSER_ACTION_SAVE,
	                                       GTK_STOCK_CANCEL,
	                                       GTK_RESPONSE_CANCEL, GTK_STOCK_SAVE,
	                                       GTK_RESPONSE_ACCEPT, NULL);
	gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (chooser), TRUE);
	gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (chooser), "cover.jpg");

	if (gtk_dialog_run (GTK_DIALOG (chooser)) == GTK_RESPONSE_ACCEPT) {
		selection = gtk_tree_view_get_selection (self->results_view);
		if (!gtk_tree_selection_get_selected (selection, &model, &iter))
			goto ret;

		gtk_tree_model_get (model, &iter, CF_RESULT_COLUMN_RESULT, &result, -1);
		source = cf_search_result_get_image (result);
		dest_path = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (chooser));
		dest = g_file_new_for_path (dest_path);

		if (!g_file_copy (source, dest, G_FILE_COPY_OVERWRITE, NULL, NULL,
		                  NULL, &error)) {
			g_printerr ("%s", error->message);
			g_error_free (error);
		}
	}

	ret:

	gtk_widget_destroy (chooser);
}

static gpointer cf_app_search_thread (gpointer data)
{
	SearchThreadData *thread_data = (SearchThreadData *) data;
	GtkWidget *widget;

	cf_provider_perform_search (thread_data->provider, thread_data->flavor_id,
	                            thread_data->search);

	gdk_threads_enter ();
	widget = GTK_WIDGET (gtk_builder_get_object (thread_data->app->builder, "SearchButton"));
	gtk_widget_set_sensitive (widget, TRUE);

	widget = GTK_WIDGET (gtk_builder_get_object (thread_data->app->builder, "CoverfinderWindow"));
	gdk_window_set_cursor (widget->window, NULL);
	gdk_cursor_destroy (thread_data->cursor);

	widget = GTK_WIDGET (thread_data->app->progress_bar);
	gtk_widget_hide (widget);

	g_object_unref (thread_data->search);
	g_free (thread_data);

	gdk_threads_leave ();

	return NULL;
}

static void cf_app_perform_search (CfApp *self)
{
	CfProvider *provider;
	CfSearch *search;
	GtkTreeIter iter;
	GtkTreeModel *model;
	GError *error = NULL;
	gint flavor_id;
	gchar *artist, *title, *keywords;
	SearchThreadData *thread_data;
	GtkWidget *widget;

	if (!gtk_combo_box_get_active_iter (self->provider_combo, &iter)) {
		return;
	}

	search = cf_search_new ();

	artist = pango_trim_string (gtk_entry_get_text (self->artist_entry));
	title = pango_trim_string (gtk_entry_get_text (self->title_entry));
	keywords = pango_trim_string (gtk_entry_get_text (self->keywords_entry));

	g_object_set (G_OBJECT(search), "title", title, "artist", artist, "keywords", keywords, NULL);

	model = gtk_combo_box_get_model (self->provider_combo);
	gtk_tree_model_get (model, &iter, 1, &provider, 2, &flavor_id, -1);

	model = gtk_tree_view_get_model (self->results_view);
	if (model != NULL) {
		gtk_tree_view_set_model (self->results_view, NULL);
		g_object_unref (G_OBJECT (model));
	}

	model = GTK_TREE_MODEL (gtk_list_store_new (CF_RESULT_COLUMN_NUM, GDK_TYPE_PIXBUF,
			G_TYPE_STRING, G_TYPE_OBJECT));
	gtk_tree_view_set_model (self->results_view, model);

	cf_provider_free_results (provider);

	widget = GTK_WIDGET (gtk_builder_get_object (self->builder, "SearchButton"));
	gtk_widget_set_sensitive (widget, FALSE);

	thread_data = g_new(SearchThreadData, 1);
	thread_data->provider = provider;
	thread_data->search = search;
	thread_data->flavor_id = flavor_id;
	thread_data->app = self;
	thread_data->cursor = gdk_cursor_new (GDK_WATCH);

	widget = GTK_WIDGET (gtk_builder_get_object (self->builder, "CoverfinderWindow"));
	gdk_window_set_cursor (widget->window, thread_data->cursor);

	widget = GTK_WIDGET (thread_data->app->progress_bar);
	gtk_widget_show (widget);

	g_thread_create(cf_app_search_thread, thread_data, FALSE, &error);
	if (error != NULL) {
		g_printerr ("Creation of thread for search failed: %s\n",
		            error->message);
		g_error_free (error);
	}
}

static void cf_app_providers_add (CfApp *self, CfProvider *provider)
{
	gchar *provider_name;
	GtkListStore *model;
	GtkTreeIter iter;

	provider_name = cf_provider_get_name (provider);
	model = GTK_LIST_STORE (gtk_combo_box_get_model (self->provider_combo));
	g_signal_connect (G_OBJECT (provider), "notify-result", G_CALLBACK (cf_app_cb_notify_result), self);
	g_signal_connect (G_OBJECT (provider), "notify-progress", G_CALLBACK (cf_app_cb_notify_progress), self);

	gtk_list_store_append (model, &iter);
	gtk_list_store_set (model, &iter, 0, provider_name, 1, provider, 2, -1, -1);
}

static void cf_app_providers_init (CfApp *self)
{
	CfProvider *provider;
	gchar *amazon_tlds[] = { "com", "co.uk", "de", "jp", "fr", "ca" };
	gchar *amazon_names[] = { "Amazon", "Amazon UK", "Amazon Germany",
	    "Amazon Japan", "Amazon France", "Amazon Canada" };
	gint i;

	for (i = 0; i < 6; i++) {
		provider = CF_PROVIDER (cf_provider_amazon_new (amazon_tlds[i], amazon_names[i]));
		cf_app_providers_add (self, provider);
	}

	gtk_combo_box_set_active (self->provider_combo, 0);
}

static void cf_app_ui_init (CfApp *self)
{
	GtkWidget *widget;
	gchar *ui_file;
	GError *error = NULL;

	self->builder = gtk_builder_new ();

	ui_file = cf_find_file ("coverfinder.ui");
	if (ui_file == NULL) {
		g_error ("The file `coverfinder.ui' cannot be found.");
		return;
	}

	gtk_builder_add_from_file (self->builder, ui_file, &error);
	g_free (ui_file);

	if (error != NULL) {
		g_error ("Loading `coverfinder.ui' failed: %s", error->message);
		return;
	}

	widget = GTK_WIDGET (gtk_builder_get_object (self->builder, "SearchLabel"));
	gtk_label_set_markup (GTK_LABEL (widget), "<b>Search</b>");
	widget = GTK_WIDGET (gtk_builder_get_object (self->builder, "SearchOptionsLabel"));
	gtk_label_set_markup (GTK_LABEL (widget), "<b>Options</b>");

	self->artist_entry = GTK_ENTRY (gtk_builder_get_object (self->builder, "SearchArtistEntry"));
	self->title_entry = GTK_ENTRY (gtk_builder_get_object (self->builder, "SearchTitleEntry"));
	self->keywords_entry = GTK_ENTRY (gtk_builder_get_object (self->builder, "SearchKeywordsEntry"));
	self->provider_combo = GTK_COMBO_BOX (gtk_builder_get_object (self->builder, "SearchOptionsDatabaseCombo"));
	self->results_view = GTK_TREE_VIEW (gtk_builder_get_object (self->builder, "ResultList"));
	self->progress_bar = GTK_PROGRESS_BAR (gtk_builder_get_object (self->builder, "Progressbar"));
	self->statusbar = GTK_STATUSBAR (gtk_builder_get_object (self->builder, "Statusbar"));
	self->statusbar_default_ctx
	    = gtk_statusbar_get_context_id (self->statusbar, "Default");

	gtk_builder_connect_signals (self->builder, self);
	gtk_tree_view_enable_model_drag_source (self->results_view,
	                                        GDK_BUTTON1_MASK, targets,
	                                        G_N_ELEMENTS (targets), GDK_ACTION_COPY);
}

static void cf_app_init (CfApp *self)
{
	cf_app_ui_init (self);
	cf_app_providers_init (self);

	self->statusbar_message = NULL;
}

void cf_app_show_window (CfApp *self)
{
	GtkWidget *window;

	window = GTK_WIDGET (gtk_builder_get_object(self->builder, "CoverfinderWindow"));
	gtk_widget_show (window);
}

void cf_app_quit (CfApp *self)
{
	gtk_main_quit ();
}

CfApp* cf_app_new (void)
{
	return g_object_new (CF_TYPE_APP, NULL);
}

/* Signal Handlers for GtkBuilder */

G_MODULE_EXPORT void cb_action_clear_cache (GtkAction *action,
                                            gpointer user_data)
{
	gchar *cache_dir;
	GFile *dir, *file;
	GFileEnumerator *enumerator = NULL;
	GError *error = NULL;
	GFileInfo *info;

	cache_dir = cf_get_cache_dir ();
	dir = g_file_new_for_path (cache_dir);
	enumerator = g_file_enumerate_children (dir, "*", G_FILE_QUERY_INFO_NONE,
	                                        NULL, &error);
	if (error != NULL) {
		g_printerr ("Enumeration of cache files failed: %s\n", error->message);
		goto ret;
	}

	while ((info = g_file_enumerator_next_file (enumerator, NULL, &error))
	    != NULL) {
		file = g_file_get_child (dir, g_file_info_get_name (info));
		if (!g_file_delete (file, NULL, &error)) {
			gchar *path;

			path = g_file_get_path (file);
			g_print ("Deleting file `%s' failed: %s\n", path, error->message);
			g_free (path);
			g_error_free (error);
			error = NULL;
		}
		g_object_unref (file);
	}

	if (error != NULL) {
		g_printerr ("Enumeration of cache files failed: %s\n", error->message);
		goto ret;
	}

	g_file_enumerator_close (enumerator, NULL, &error);
	if (error != NULL) {
		g_printerr ("Closing of enumeration of cache files failed: %s\n",
		            error->message);
		goto ret;
	}

	ret:

	if (error != NULL)
		g_error_free (error);
	g_free (cache_dir);
	g_object_unref (dir);
	g_free (error);
}

G_MODULE_EXPORT void cb_action_about (GtkAction *action, gpointer user_data)
{
	GtkWidget *dialog;
	CfApp *self = CF_APP (user_data);

	dialog = GTK_WIDGET (gtk_builder_get_object (self->builder, "CoverfinderAboutDialog"));
	gtk_about_dialog_set_version (GTK_ABOUT_DIALOG (dialog), VERSION);
	gtk_dialog_run (GTK_DIALOG (dialog));
	gtk_widget_hide (dialog);
}

G_MODULE_EXPORT void cb_action_save (GtkAction *action, gpointer user_data)
{
	cf_app_save_selection (CF_APP (user_data));
}

G_MODULE_EXPORT void cb_action_quit (GtkAction *action, gpointer user_data)
{
	cf_app_quit (CF_APP (user_data));
}

G_MODULE_EXPORT gboolean cb_window_delete (GtkWidget *widget,
                                           GdkEvent *event,
                                           gpointer user_data)
{
	cf_app_quit (CF_APP (user_data));
	return TRUE;
}

G_MODULE_EXPORT void cb_search_button_activate (GtkButton *widget,
                                                gpointer user_data)
{
	cf_app_perform_search (CF_APP (user_data));
}

G_MODULE_EXPORT gboolean cb_button_pressed (GtkWidget *widget,
                                            GdkEventButton *event,
                                            gpointer user_data)
{
	CfApp *self = CF_APP (user_data);
	GtkMenu *menu;

	if (event->button == 3) {
		menu = GTK_MENU (gtk_builder_get_object (self->builder, "ResultPopup"));
		gtk_menu_popup (menu, NULL, NULL, NULL, user_data, event->button,
		                gtk_get_current_event_time ());
	}
	return FALSE;
}

G_MODULE_EXPORT void cb_result_data_get (GtkWidget *widget,
                                         GdkDragContext *drag_context,
                                         GtkSelectionData *data,
                                         guint info,
                                         guint time,
                                         gpointer user_data)
{
	GtkTreeSelection *selection;
	GtkTreeModel *model;
	GtkTreeIter iter;
	CfSearchResult *result;
	GFile *source;
	gchar *uri;

	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (widget));
	if (!gtk_tree_selection_get_selected (selection, &model, &iter))
		return;

	gtk_tree_model_get (model, &iter, CF_RESULT_COLUMN_RESULT, &result, -1);
	source = cf_search_result_get_image (result);
	uri = g_file_get_uri (source);
	gtk_selection_data_set (data, data->target, 8, (guchar*) uri, strlen (uri));
	g_free (uri);
}
