/*
 * CoverFinder - Find and download cover images from Amazon
 * Copyright (C) 2007 - 2008 - Sven Salzwedel
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
#include "cf-search-result-amazon.h"
#include "cf-search-result.h"

enum
{
	RESULT_NOPROP,
	RESULT_ARTIST,
	RESULT_TITLE,
	RESULT_TYPE,
	RESULT_IMAGE,
	RESULT_IMAGE_URI
};

static void cf_search_result_amazon_get_property (GObject *object,
	guint property_id, GValue *value, GParamSpec *pspec)
{
	CfSearchResultAmazon *self = CF_SEARCH_RESULT_AMAZON (object);

	switch (property_id) {
	case RESULT_ARTIST:
		g_value_set_string (value, self->artist);
		break;
	case RESULT_TITLE:
		g_value_set_string (value, self->title);
		break;
	case RESULT_TYPE:
		g_value_set_int (value, self->type);
		break;
	case RESULT_IMAGE:
		g_value_set_object (value, self->image);
		break;
	case RESULT_IMAGE_URI:
		g_value_set_string (value, self->image_uri);
		break;
	default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
	}
}

static void cf_search_result_amazon_set_property (GObject *object,
	guint property_id, const GValue *value, GParamSpec *pspec)
{
	CfSearchResultAmazon *self = CF_SEARCH_RESULT_AMAZON (object);

	switch (property_id) {
	case RESULT_ARTIST:
		self->artist = g_value_dup_string (value);
		break;
	case RESULT_TITLE:
		self->title = g_value_dup_string (value);
		break;
	case RESULT_TYPE:
		self->type = g_value_get_int (value);
		break;
	case RESULT_IMAGE:
		self->image = G_FILE (g_value_get_object (value));
		break;
	case RESULT_IMAGE_URI:
		self->image_uri = g_value_dup_string (value);
		break;
	default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
	}
}

static gchar *cf_search_result_amazon_get_description (CfSearchResult *iface)
{
	CfSearchResultAmazon *self = CF_SEARCH_RESULT_AMAZON (iface);
	GString *markup;
	gchar *escaped;

	markup = g_string_new (NULL);

	escaped = g_markup_escape_text (self->title, -1);
	g_string_append_printf (markup,
		"<span size=\"larger\" weight=\"heavy\">%s</span>", escaped);
	g_free (escaped);
	if (self->artist != NULL) {
		escaped = g_markup_escape_text (self->artist, -1);
		g_string_append_printf (markup, "\nby <i>%s</i>", escaped);
		g_free (escaped);
	}
	switch (self->type) {
	case AMAZON_TYPE_AUDIOCD:
		g_string_append_printf (markup, "\n\n%s", "Audio CD");
		break;
	case AMAZON_TYPE_DVD:
		g_string_append_printf (markup, "\n\n%s", "DVD");
		break;
	case AMAZON_TYPE_LP:
		g_string_append_printf (markup, "\n\n%s", "LP");
		break;
	}

	return g_string_free (markup, FALSE);
}

GFile *cf_search_result_amazon_get_image (CfSearchResult *iface)
{
	CfSearchResultAmazon *self = CF_SEARCH_RESULT_AMAZON (iface);

	return self->image;
}

static void cf_search_result_amazon_interface_init (gpointer g_iface,
	gpointer iface_data)
{
	CfSearchResultInterface *iface = (CfSearchResultInterface *) g_iface;

	iface->get_description = cf_search_result_amazon_get_description;
	iface->get_image = cf_search_result_amazon_get_image;
}

/* object/class initialization */

G_DEFINE_TYPE_EXTENDED (CfSearchResultAmazon, cf_search_result_amazon,
	G_TYPE_OBJECT, 0, G_IMPLEMENT_INTERFACE (CF_TYPE_SEARCH_RESULT,
		cf_search_result_amazon_interface_init))

static void cf_search_result_amazon_finalize (GObject *object)
{
	CfSearchResultAmazon *self = CF_SEARCH_RESULT_AMAZON (object);

	if (G_OBJECT_CLASS (cf_search_result_amazon_parent_class)->finalize)
		G_OBJECT_CLASS (cf_search_result_amazon_parent_class)->finalize (object);

	g_free (self->artist);
	g_free (self->title);
	if (self->image != NULL)
		g_object_unref (G_OBJECT (self->image));
}

static void cf_search_result_amazon_class_init (
	CfSearchResultAmazonClass *klass)
{
	GObjectClass *object_class = G_OBJECT_CLASS (klass);
	GParamSpec *spec;

	object_class->get_property = cf_search_result_amazon_get_property;
	object_class->set_property = cf_search_result_amazon_set_property;
	object_class->finalize = cf_search_result_amazon_finalize;

	spec = g_param_spec_string ("artist", NULL, NULL, NULL, G_PARAM_READWRITE);
	g_object_class_install_property (object_class, RESULT_ARTIST, spec);

	spec = g_param_spec_string ("title", NULL, NULL, NULL, G_PARAM_READWRITE);
	g_object_class_install_property (object_class, RESULT_TITLE, spec);

	spec = g_param_spec_int ("type", NULL, NULL, 0, AMAZON_TYPE_NUM_ELEMENTS
		- 1, 0, G_PARAM_READWRITE);
	g_object_class_install_property (object_class, RESULT_TYPE, spec);

	spec = g_param_spec_object ("image", NULL, NULL, G_TYPE_FILE,
		G_PARAM_READWRITE);
	g_object_class_install_property (object_class, RESULT_IMAGE, spec);

	spec = g_param_spec_string ("image-uri", NULL, NULL, NULL,
		G_PARAM_READWRITE);
	g_object_class_install_property (object_class, RESULT_IMAGE_URI, spec);
}

static void cf_search_result_amazon_init (CfSearchResultAmazon *self)
{
	self->artist = NULL;
	self->image = NULL;
	self->title = NULL;
	self->type = AMAZON_TYPE_OTHER;
}

CfSearchResultAmazon*
cf_search_result_amazon_new (void)
{
	return g_object_new (CF_TYPE_SEARCH_RESULT_AMAZON, NULL);
}
